/*
** Copyright 1998 - 2005 Double Precision, Inc.  See COPYING for
** distribution information.
*/

#if HAVE_CONFIG_H
#include "courier_auth_config.h"
#endif
#include	<stdio.h>
#include	<stdlib.h>
#include	<ctype.h>
#include	<string.h>
#include	<errno.h>
#include	<unistd.h>

#include	"authpipe.h"
#include	"courierauth.h"
#include	"authstaticlist.h"
#include	"courierauthdebug.h"

#include	"authpipelib.h"
#include	"authpiperc.h"

static int disabled_flag;

static int auth_pipe_login(const char *service, char *authdata,
			     int (*callback_func)(struct authinfo *, void *),
			     int *callback_arg)
{
	const char *user, *pass;

	if ((user=strtok(authdata, "\n")) == 0 ||
		(pass=strtok(0, "\n")) == 0)
	{
		errno=EPERM;
		return (0);
	}

	return authpipecommon(user, pass, callback_func, callback_arg);
}

#if HAVE_HMACLIB

#include	"libhmac/hmac.h"
#include	"cramlib.h"

static int auth_pipe_cram(const char *service,
			    const char *authtype,
			    char *authdata,
			    int (*callback_func)(struct authinfo *, void *),
			    void *callback_arg)
{
	struct	cram_callback_info	cci;

	if (auth_get_cram(authtype, authdata, &cci))
		return (-1);

	cci.callback_func=callback_func;
	cci.callback_arg=callback_arg;

	return authpipecommon(cci.user, 0, &auth_cram_callback, &cci);
}
#endif

int auth_pipe(const char *service, const char *authtype, char *authdata,
		int (*callback_func)(struct authinfo *, void *),
		void *callback_arg)
{
	if (disabled_flag)
		return -1;

	if (strcmp(authtype, AUTHTYPE_LOGIN) == 0)
		return (auth_pipe_login(service, authdata,
			callback_func, callback_arg));

#if HAVE_HMACLIB
	return (auth_pipe_cram(service, authtype, authdata,
				 callback_func, callback_arg));
#else
	return -1;
#endif
}


int auth_pipe_pre(const char *userid, const char *service,
		int (*callback)(struct authinfo *, void *),
		void *arg)
{
	if (disabled_flag)
		return -1;

	return (authpipecommon(userid, 0, callback, arg));
}


/* service is (for now) ignored */
int auth_pipe_chgpwd(const char *service,
		const char *uid,
		const char *oldpwd,
		const char *newpwd)
{
#define STRING_BUFFER_SIZE 1024
#define SBSM1 STRING_BUFFER_SIZE - 1
	/* fds */
	int dataIn, dataOut;
	char response[STRING_BUFFER_SIZE];
	int rc;

	if (disabled_flag)
	{
		errno=EPERM;
		return -1;
	}

	errno = 0;

	DPWPRINTF("pipe: will change password of uid %s from %s to %s: ",
			uid, oldpwd, newpwd);

	if (forkPipe(&dataIn, &dataOut)) goto FAILED;

	if (prog2pipe(dataOut, dataIn, "PASSWD\n") <= 0 ||
	    prog2pipe(dataOut, dataIn, uid) <= 0 ||
	    prog2pipe(dataOut, dataIn, "\n") <= 0 ||
	    prog2pipe(dataOut, dataIn, oldpwd) <= 0 ||
	    prog2pipe(dataOut, dataIn, "\n") <= 0 ||
	    prog2pipe(dataOut, dataIn, newpwd) <= 0 ||
	    prog2pipe(dataOut, dataIn, "\n") <= 0) goto FAILED;

	if ((rc = pipe2prog(dataOut, dataIn, response, SBSM1)) <= 0)
	{
		/* either 'OK ' did not come, or pipe broke */
		if (rc < 0) goto FAILED;
		else if (rc == 2 && response[0] == 'O' && response[1] == 'K')
		{
			/* do nothing pipe should have answered with 'OK '
			   but returned only 'OK' */
		}
		else goto PW_ERROR;
	}
		
	DPWPRINTF("OK\n");
	closePipe(dataIn, dataOut);
	return 0;

FAILED:
	errno = EINVAL;
	/* no break,... */
PW_ERROR:
	if (! errno) errno = EPERM;
	DPWPRINTF("FAILED\n");
	DPWPRINTF("pipeauth: could not change pwd for %s: %s\n",
		uid, response);
	return 1;
}


void auth_pipe_idle()
{
	/* don't need to do anything when idle */
}


void auth_pipe_close()
{
	/* don't need to do anything when closing */
}


/* not yet implemented */
void auth_pipe_enumerate(void(*cb_func)(const char *name,
	                                      uid_t uid,
	                                      gid_t gid,
	                                const char *homedir,
	                                const char *maildir,
					const char *options,
	                                      void *void_arg),
	                         void *void_arg)
{
	/* Insert code */
}


static struct authstaticinfo authpipe_info={
	"authpipe",
	auth_pipe,
	auth_pipe_pre,
	auth_pipe_close,
	auth_pipe_chgpwd,
	auth_pipe_idle}; /*,
	auth_pipe_enumerate}; */


struct authstaticinfo *courier_authpipe_init()
{
	disabled_flag=access(PIPE_PROGRAM, X_OK);
	return &authpipe_info;
}

