/*
** Copyright 1998 - 2000 Double Precision, Inc.  See COPYING for
** distribution information.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <unistd.h>
/* for fork */
#include <sys/types.h>
/*  used to avoid zombies */
#include <signal.h>
#include <sys/wait.h>


#include	"auth.h"
#include	"authcustom.h"
#include	"courierauthdebug.h"

#include	"courierauthdebug.h"

#include	"authpipelib.h"
#include	"authpiperc.h"


static void removeZombie(int signum)
{
	wait(NULL);
}

static void execChild(int to[], int from[])
{
	DPRINTF("executing %s", PIPE_PROGRAM);

	close(STDIN_FILENO); dup2(to[0], STDIN_FILENO);
	close(STDOUT_FILENO); dup2(from[1], STDOUT_FILENO);
	close(to[0]); close(to[1]); close(from[0]); close(from[1]);

	/* avoid zombies */
	signal(SIGCHLD, removeZombie);

	execl(PIPE_PROGRAM, PIPE_PROGRAM, NULL);

	DPRINTF("pipe: failed to execute %s: %s",PIPE_PROGRAM, strerror(errno));
	exit(1);
}


/* returns -1 for error, >0 for OK
   closes fds on error! */
int prog2pipe(int to, int from, const char *text)
{
	int stringLength = strlen(text);
	int tmpSL = stringLength;
	int charsSent;
	int hadError = 0;
	const char *origText = text;

	DPWPRINTF("pipe: will send following to pipe: ##%s##", origText);

	tmpSL = stringLength;
	while (!hadError)
	{
		charsSent = write(to, text, tmpSL);
		tmpSL -= charsSent;
		if (charsSent <= 0) hadError = 1;
		else if (tmpSL == 0) break; /* sent everything */
		else text = &text[charsSent];
	}

	if(hadError)
	{
		DPWPRINTF(" FAILED\n");
		DPWPRINTF("pipe: could not write to pipe: %s", strerror(errno));
		close(to); close(from);
		return -1;
	}

	DPWPRINTF(" OK\n");
	return stringLength;
}

/* returns -1 for error, length of text for OK
   closes fds on error! */
int pipe2prog(int to, int from, char *text, int readMax)
{
	int charsRead;
	int hadError = 0;
	int stringLength = 0;
	char *origText = text;
	
	while (!hadError)
	{
		charsRead = read(from, text, readMax);
		stringLength += charsRead;
		if (charsRead <= 0) hadError = 1;
		else if (text[charsRead - 1] == '\n')  /* got everything */
		{
			text[charsRead - 1] = '\0';
			break;
		}
		else
		{
			readMax -= charsRead;
			text = &text[charsRead];
		}
	}
	if (hadError)
	{
		DPRINTF("pipe: could not read from pipe: %s",
		        strerror(errno));
		close(to); close(from);
		return -1;
	}

	DPRINTF("pipe: received following from pipe: ##%s##", origText);
	return stringLength;
}


int getFromPipe(int to, int from, const char *what, char *answer, int readMax)
{
	int ret;

	if (prog2pipe(to, from, what) <= 0 || 
	    prog2pipe(to, from, "\n") <= 0 || 
	    (ret = pipe2prog(to, from, answer, readMax)) <= 0) return -1;

	if (ret < 3 ||
	    !((answer[0] == '?' &&
	       answer[1] == '?') ||
	      (answer[0] == 'O' &&
	       answer[1] == 'K')) ||
	    answer[2] != ' ') return 0;

	return ret;
}

void closePipe(int dataIn, int dataOut)
{
	close(dataIn);
	close(dataOut);
}

int forkPipe(int *dataIn, int *dataOut)
{
	int to[2], from[2];
	pid_t child;

	/* let's create 2 pipes */
	if(pipe(to) < 0) {
		DPRINTF("pipe: failed to create pipe: %s", strerror(errno));
		return 1;
	}

	if(pipe(from) < 0) {
		DPRINTF("pipe: failed to create pipe: %s", strerror(errno));
		close(to[0]); close(to[1]);
		return 1;
	}

	DPRINTF("attempting to fork");
	child = fork();
	if(child < 0) {
		DPRINTF("pipe: failed to fork: %s", strerror(errno));
		close(to[0]); close(to[1]); close(from[0]); close(from[1]);
		return 1;
	}

	/* child */
	if(child == 0) execChild(to, from);

	signal(SIGCHLD, removeZombie);

	/* parent */
	DPRINTF("Pipe auth. started Pipe-program (pid %d)", child);

	close(to[0]); close(from[1]);
	*dataIn = from[0]; *dataOut = to[1];
	return 0;
}

